from typing import Optional

from .base_model import BaseModel
from .fragments import ConsumptionFragment


class Consumption(BaseModel):
    consumption: Optional["ConsumptionConsumption"]


class ConsumptionConsumption(ConsumptionFragment):
    pass


Consumption.model_rebuild()
ConsumptionConsumption.model_rebuild()



from .active_coop_status import ActiveCoopStatus, ActiveCoopStatusCoop
from .all_coops import AllCoops, AllCoopsAllCoops
from .async_base_client import AsyncBaseClient
from .authenticate import Authenticate, AuthenticateTokenAuth
from .base_model import BaseModel, Upload
from .client import Client
from .consumption import Consumption, ConsumptionConsumption
from .coop import Coop, CoopCoop
from .coop_timeline_progression import (
    CoopTimelineProgression,
    CoopTimelineProgressionCoopTimelineProgression,
)
from .cumulative_savings import (
    CumulativeSavings,
    CumulativeSavingsCumulativeSavingsData,
    CumulativeSavingsCumulativeSavingsDataCurrency,
)
from .deauthenticate import (
    Deauthenticate,
    DeauthenticateAuthLogoutSession,
    DeauthenticateDeleteRefreshTokenCookie,
    DeauthenticateDeleteTokenCookie,
)
from .enums import (
    AccountType,
    ApprovalStatus,
    BusinessSector,
    BusinessType,
    CardBrandType,
    CoopCoopStatusChoices,
    CrmUserMemoChannelChoices,
    GenerationDataTitle,
    GenerationGenerationFarmGenerationTypeChoices,
    GenerationGenerationFarmOperationalStatusChoices,
    GetMemberOctopusConsumptionDataTitle,
    InvoiceType,
    MembersMemberConsumptionEvidenceSubmissionStatusChoices,
    PaymentsPaymentCategoryChoices,
    PaymentStatus,
    PaymentType,
    Period,
    StripePaymentType,
    SurveyQuestionChoiceTypeChoices,
)
from .exceptions import (
    GraphQLClientError,
    GraphQLClientGraphQLError,
    GraphQLClientGraphQLMultiError,
    GraphQLClientHttpError,
    GraphQlClientInvalidResponseError,
)
from .faqs import Faqs, FaqsFaqs, FaqsFaqsTags
from .fragments import (
    ConsumptionFragment,
    CoopFragment,
    CoopFragmentCurrency,
    CoopFragmentDocuments,
    CoopFragmentDocumentsDocument,
    CoopFragmentDocumentsDocumentDocumentTags,
    CoopFragmentGenerationfarm,
    CoopFragmentGenerationfarmCurrency,
    MemberFragment,
    MemberFragmentAddress,
    MemberFragmentBadges,
    MemberFragmentBeneficiaries,
    MemberFragmentCurrentMemberSupplier,
    MemberFragmentCurrentMemberSupplierSupplier,
    MemberFragmentInvoices,
    MemberFragmentInvoicesQuote,
    MemberFragmentInvoicesQuoteCurrency,
    MemberFragmentMemberDocument,
    MemberFragmentMemberships,
    MemberFragmentMembershipsCoop,
    MemberFragmentWaitingListPlaces,
    WaitingListFragment,
    WaitingListFragmentCoop,
    WaitingListPlaceFragment,
    WaitingListPlaceFragmentWaitingList,
)
from .input_types import (
    AddAdditionalWattsInputs,
    AddMemberToCoopWaitingListInput,
    AddressInput,
    ApproveMemberConsumptionEvidenceSubmissionInput,
    AuthenticationCreateAccountInput,
    AuthLoginSessionInputType,
    BillingAddress,
    CalculateOrderPaymentInput,
    CalculatePaymentLineInput,
    CalculateUnitsForCostInput,
    ChangeUserPasswordInput,
    ClientEnvInput,
    ConsumptionEvidenceInput,
    ConsumptionInput,
    CreateAccountInput,
    CreateContactUsMessageInput,
    CreateEmployerContactInput,
    CreateMemberBeneficiaryInput,
    CreateMemberConsumptionEvidenceSubmissionInput,
    CreateOrUpdateNewsInput,
    CreateOwnershipUserInput,
    CreateReservedUserInput,
    CreateUserMemoInputs,
    DirectDebitInput,
    FilterInput,
    GetKeyValuePair,
    GetMemberMissingInvoiceDataInputs,
    GiftCardInput,
    InsightsChartDataInput,
    InstalmentInput,
    OctopusApiAddressInput,
    OrderInput,
    PaginationInput,
    PasswordResetConfirmInput,
    PayCapacityQuoteInput,
    PaymentInput,
    PaymentLineInput,
    PayOrderInput,
    PayScheduledPaymentInput,
    QuoteInput,
    RecipientAddressInput,
    RecipientInput,
    RegistrationUpdateUserPersonalDetailsInput,
    RequestCallBackInput,
    ResponseInputType,
    SearchAndFilterInput,
    SearchCountAndFilterInput,
    SearchCountInput,
    SearchInput,
    SenderInput,
    SingleFileUploadInput,
    SingleNewsFileUploadInput,
    StripePaymentInput,
    SupplierMemberInfoInput,
    TokenAuthenticationInput,
    UpdateMemberBeneficiaryInput,
    UpdateMemberConsumptionEvidenceSubmissionAdminNoteInput,
    UpdateQuoteInput,
    UpdateSupplierQuotesInput,
    UpdateUserCRMInput,
    UpdateUserInput,
    UserContactInput,
    UserInput,
)
from .me import (
    Me,
    MeMe,
    MeMeDirectDebit,
    MeMeGroups,
    MeMeGroupsPermissions,
    MeMeMember,
    MeMePayments,
    MeMeReferred,
    MeMeReferredRecommendedBy,
    MeMeReferredUser,
    MeMeReferredUserMember,
)
from .member import Member, MemberMember
from .monthly_savings import (
    MonthlySavings,
    MonthlySavingsMonthlySavingsData,
    MonthlySavingsMonthlySavingsDataCurrency,
)
from .refresh_token import RefreshToken, RefreshTokenRefreshToken
from .tribe_url import TribeUrl
from .verify_token import VerifyToken, VerifyTokenVerifyToken
from .version import Version
from .wind_farm_generation import (
    WindFarmGeneration,
    WindFarmGenerationMember,
    WindFarmGenerationMemberMemberships,
    WindFarmGenerationMemberMembershipsCoop,
    WindFarmGenerationMemberMembershipsCoopCurrency,
    WindFarmGenerationMemberMembershipsCoopGenerationfarm,
    WindFarmGenerationMemberMembershipsCoopGenerationfarmGenerationData,
    WindFarmGenerationMemberMembershipsCoopGenerationfarmGenerationDataDataSet,
)

__all__ = [
    "AccountType",
    "ActiveCoopStatus",
    "ActiveCoopStatusCoop",
    "AddAdditionalWattsInputs",
    "AddMemberToCoopWaitingListInput",
    "AddressInput",
    "AllCoops",
    "AllCoopsAllCoops",
    "ApprovalStatus",
    "ApproveMemberConsumptionEvidenceSubmissionInput",
    "AsyncBaseClient",
    "AuthLoginSessionInputType",
    "Authenticate",
    "AuthenticateTokenAuth",
    "AuthenticationCreateAccountInput",
    "BaseModel",
    "BillingAddress",
    "BusinessSector",
    "BusinessType",
    "CalculateOrderPaymentInput",
    "CalculatePaymentLineInput",
    "CalculateUnitsForCostInput",
    "CardBrandType",
    "ChangeUserPasswordInput",
    "Client",
    "ClientEnvInput",
    "Consumption",
    "ConsumptionConsumption",
    "ConsumptionEvidenceInput",
    "ConsumptionFragment",
    "ConsumptionInput",
    "Coop",
    "CoopCoop",
    "CoopCoopStatusChoices",
    "CoopFragment",
    "CoopFragmentCurrency",
    "CoopFragmentDocuments",
    "CoopFragmentDocumentsDocument",
    "CoopFragmentDocumentsDocumentDocumentTags",
    "CoopFragmentGenerationfarm",
    "CoopFragmentGenerationfarmCurrency",
    "CoopTimelineProgression",
    "CoopTimelineProgressionCoopTimelineProgression",
    "CreateAccountInput",
    "CreateContactUsMessageInput",
    "CreateEmployerContactInput",
    "CreateMemberBeneficiaryInput",
    "CreateMemberConsumptionEvidenceSubmissionInput",
    "CreateOrUpdateNewsInput",
    "CreateOwnershipUserInput",
    "CreateReservedUserInput",
    "CreateUserMemoInputs",
    "CrmUserMemoChannelChoices",
    "CumulativeSavings",
    "CumulativeSavingsCumulativeSavingsData",
    "CumulativeSavingsCumulativeSavingsDataCurrency",
    "Deauthenticate",
    "DeauthenticateAuthLogoutSession",
    "DeauthenticateDeleteRefreshTokenCookie",
    "DeauthenticateDeleteTokenCookie",
    "DirectDebitInput",
    "Faqs",
    "FaqsFaqs",
    "FaqsFaqsTags",
    "FilterInput",
    "GenerationDataTitle",
    "GenerationGenerationFarmGenerationTypeChoices",
    "GenerationGenerationFarmOperationalStatusChoices",
    "GetKeyValuePair",
    "GetMemberMissingInvoiceDataInputs",
    "GetMemberOctopusConsumptionDataTitle",
    "GiftCardInput",
    "GraphQLClientError",
    "GraphQLClientGraphQLError",
    "GraphQLClientGraphQLMultiError",
    "GraphQLClientHttpError",
    "GraphQlClientInvalidResponseError",
    "InsightsChartDataInput",
    "InstalmentInput",
    "InvoiceType",
    "Me",
    "MeMe",
    "MeMeDirectDebit",
    "MeMeGroups",
    "MeMeGroupsPermissions",
    "MeMeMember",
    "MeMePayments",
    "MeMeReferred",
    "MeMeReferredRecommendedBy",
    "MeMeReferredUser",
    "MeMeReferredUserMember",
    "Member",
    "MemberFragment",
    "MemberFragmentAddress",
    "MemberFragmentBadges",
    "MemberFragmentBeneficiaries",
    "MemberFragmentCurrentMemberSupplier",
    "MemberFragmentCurrentMemberSupplierSupplier",
    "MemberFragmentInvoices",
    "MemberFragmentInvoicesQuote",
    "MemberFragmentInvoicesQuoteCurrency",
    "MemberFragmentMemberDocument",
    "MemberFragmentMemberships",
    "MemberFragmentMembershipsCoop",
    "MemberFragmentWaitingListPlaces",
    "MemberMember",
    "MembersMemberConsumptionEvidenceSubmissionStatusChoices",
    "MonthlySavings",
    "MonthlySavingsMonthlySavingsData",
    "MonthlySavingsMonthlySavingsDataCurrency",
    "OctopusApiAddressInput",
    "OrderInput",
    "PaginationInput",
    "PasswordResetConfirmInput",
    "PayCapacityQuoteInput",
    "PayOrderInput",
    "PayScheduledPaymentInput",
    "PaymentInput",
    "PaymentLineInput",
    "PaymentStatus",
    "PaymentType",
    "PaymentsPaymentCategoryChoices",
    "Period",
    "QuoteInput",
    "RecipientAddressInput",
    "RecipientInput",
    "RefreshToken",
    "RefreshTokenRefreshToken",
    "RegistrationUpdateUserPersonalDetailsInput",
    "RequestCallBackInput",
    "ResponseInputType",
    "SearchAndFilterInput",
    "SearchCountAndFilterInput",
    "SearchCountInput",
    "SearchInput",
    "SenderInput",
    "SingleFileUploadInput",
    "SingleNewsFileUploadInput",
    "StripePaymentInput",
    "StripePaymentType",
    "SupplierMemberInfoInput",
    "SurveyQuestionChoiceTypeChoices",
    "TokenAuthenticationInput",
    "TribeUrl",
    "UpdateMemberBeneficiaryInput",
    "UpdateMemberConsumptionEvidenceSubmissionAdminNoteInput",
    "UpdateQuoteInput",
    "UpdateSupplierQuotesInput",
    "UpdateUserCRMInput",
    "UpdateUserInput",
    "Upload",
    "UserContactInput",
    "UserInput",
    "VerifyToken",
    "VerifyTokenVerifyToken",
    "Version",
    "WaitingListFragment",
    "WaitingListFragmentCoop",
    "WaitingListPlaceFragment",
    "WaitingListPlaceFragmentWaitingList",
    "WindFarmGeneration",
    "WindFarmGenerationMember",
    "WindFarmGenerationMemberMemberships",
    "WindFarmGenerationMemberMembershipsCoop",
    "WindFarmGenerationMemberMembershipsCoopCurrency",
    "WindFarmGenerationMemberMembershipsCoopGenerationfarm",
    "WindFarmGenerationMemberMembershipsCoopGenerationfarmGenerationData",
    "WindFarmGenerationMemberMembershipsCoopGenerationfarmGenerationDataDataSet",
]



from typing import Optional

from .base_model import BaseModel
from .enums import CoopCoopStatusChoices


class ActiveCoopStatus(BaseModel):
    coop: Optional["ActiveCoopStatusCoop"]


class ActiveCoopStatusCoop(BaseModel):
    id: str
    status: CoopCoopStatusChoices


ActiveCoopStatus.model_rebuild()
ActiveCoopStatusCoop.model_rebuild()



from typing import Optional

from .base_model import BaseModel
from .fragments import MemberFragment


class Member(BaseModel):
    member: Optional["MemberMember"]


class MemberMember(MemberFragment):
    pass


Member.model_rebuild()
MemberMember.model_rebuild()



from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class MonthlySavings(BaseModel):
    monthly_savings_data: Optional["MonthlySavingsMonthlySavingsData"] = Field(
        alias="monthlySavingsData"
    )


class MonthlySavingsMonthlySavingsData(BaseModel):
    currency: "MonthlySavingsMonthlySavingsDataCurrency"
    month: str
    year: str
    savings: float
    referred_savings: Optional[float] = Field(alias="referredSavings")
    generation: float
    co2_saved: float = Field(alias="co2Saved")
    tree_equivalent: int = Field(alias="treeEquivalent")


class MonthlySavingsMonthlySavingsDataCurrency(BaseModel):
    code: str
    precision: int


MonthlySavings.model_rebuild()
MonthlySavingsMonthlySavingsData.model_rebuild()
MonthlySavingsMonthlySavingsDataCurrency.model_rebuild()



from __future__ import annotations

import logging
from datetime import datetime
from types import TracebackType
from typing import List, Optional, Type

from pydantic import validate_call

from .constants import RIPPLE_GRAPH_URL
from .exceptions import (
    RippleEnergyAuthenticationException,
    RippleEnergyDeauthenticationException,
    RippleEnergyMissingAuthorizationHeaderException,
    RippleEnergyTokenDestroyException,
)
from .graphql_client import (
    ActiveCoopStatusCoop,
    AllCoopsAllCoops,
    AuthenticateTokenAuth,
    Client,
    ConsumptionConsumption,
    CoopCoop,
    CoopTimelineProgressionCoopTimelineProgression,
    CumulativeSavingsCumulativeSavingsData,
    Deauthenticate,
    FaqsFaqs,
    MemberMember,
    MeMe,
    MonthlySavingsMonthlySavingsData,
    RefreshTokenRefreshToken,
    TokenAuthenticationInput,
    VerifyTokenVerifyToken,
    WindFarmGenerationMember,
)
from .helpers import check_expiry, generate_jwt_header
from .models import RippleEnergyCredentialAuth, RippleEnergyTokenAuth

logger = logging.getLogger(__name__)


class RippleEnergy:
    @validate_call
    def __init__(
        self,
        auth: RippleEnergyCredentialAuth | RippleEnergyTokenAuth,
        auto_auth_deauth: bool = True,
    ):
        """Initialise Ripple Energy object"""
        if isinstance(auth, RippleEnergyCredentialAuth):
            self.auth_method = "credential"
            self.email = auth.email
            self.password = auth.password
            self.headers: dict[str, str] = {}
            self.token = None
        elif isinstance(auth, RippleEnergyTokenAuth):
            self.auth_method = "token"
            self.token = auth.token
            self.headers = generate_jwt_header(auth.token)

        logger.info(f"Using authentication method: {self.auth_method}")

        self.auto_auth_deauth = auto_auth_deauth
        self.client = Client(url=RIPPLE_GRAPH_URL, headers=self.headers)

    async def __aenter__(self) -> RippleEnergy:
        """Ripple Energy asyncronous enter"""
        if self.auto_auth_deauth:
            if self.auth_method == "credential":
                await self.authenticate()
        elif self.auth_method == "token":
            await self.verify_token()

        return self

    async def __aexit__(
        self,
        exception_type: Optional[Type[BaseException]],
        exception_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> None:
        """Ripple Energy asyncronous exit"""
        if self.auto_auth_deauth:
            await self.deauthenticate()

    async def authenticate(self) -> AuthenticateTokenAuth:
        """Authenticate with Ripple Energy and generate JWT token"""
        input = TokenAuthenticationInput(email=self.email, password=self.password)

        data = await self.client.authenticate(input=input)

        logger.debug(f"Authentication response: {data}")

        if not data.token:
            raise RippleEnergyAuthenticationException

        logger.info(f"Authentication successful. Token: {data.token}")

        self.token = data.token
        self.headers.update(generate_jwt_header(data.token))

        await self.verify_token()

        return data

    async def deauthenticate(self) -> Deauthenticate:
        """De-authenticate with Ripple Energy and destroy JWT token"""
        if not self.headers.get("Authorization"):
            raise RippleEnergyMissingAuthorizationHeaderException

        data = await self.client.deauthenticate()

        logger.debug(f"De-authentication response: {data}")

        if data.auth_logout_session and not data.auth_logout_session.logout_successful:
            raise RippleEnergyDeauthenticationException
        if data.delete_token_cookie and not data.delete_token_cookie.deleted:
            raise RippleEnergyTokenDestroyException

        self.token = None
        self.token_expires = datetime.now()
        self.headers.pop("Authorization")

        return data

    async def refresh_token(self) -> RefreshTokenRefreshToken | None:
        """Ripple Energy refresh JWT token"""
        data = await self.client.refresh_token(self.token)

        logger.debug(f"Refresh token response: {data}")

        if isinstance(data, RefreshTokenRefreshToken):
            logger.info(f"Token refresh successful. Token: {data.token}")

            self.token = data.token
            self.headers.update(generate_jwt_header(data.token))
            self.token_expires = datetime.fromtimestamp(data.refresh_expires_in)
        else:
            raise RippleEnergyAuthenticationException

        return data

    async def verify_token(self) -> VerifyTokenVerifyToken | None:
        """Ripple Energy verify JWT token"""
        data = await self.client.verify_token(self.token)

        logger.debug(f"Verify token response: {data}")

        if isinstance(data, VerifyTokenVerifyToken):
            self.token_expires = datetime.fromtimestamp(data.payload["exp"])

        logger.info(f"Token verified. Expires: {self.token_expires}")

        return data

    @check_expiry
    async def version(self) -> str | None:
        """Ripple Energy GraphQL API version"""
        data = await self.client.version()

        logger.debug(f"Version response: {data}")

        return data

    @check_expiry
    async def member(self) -> MemberMember | None:
        """Ripple Energy member data"""
        data = await self.client.member()

        logger.debug(f"Member response: {data}")

        return data

    @check_expiry
    async def me(self) -> MeMe | None:
        """Ripple Energy user data"""
        data = await self.client.me()

        logger.debug(f"Me response: {data}")

        return data

    @check_expiry
    async def active_coop_status(self) -> ActiveCoopStatusCoop | None:
        """Ripple Energy active co-op status"""
        data = await self.client.active_coop_status()

        logger.debug(f"Active co-op status response: {data}")

        return data

    @check_expiry
    async def coop(self) -> CoopCoop | None:
        """Ripple Energy co-op"""
        data = await self.client.coop()

        logger.debug(f"Co-op response: {data}")

        return data

    @check_expiry
    async def tribe_url(self) -> str:
        """Ripple Energy Tribe URL"""
        data = await self.client.tribe_url()

        logger.debug(f"Tribe URL response: {data}")

        return data

    @check_expiry
    @validate_call
    async def faqs(self, tag: str = "") -> List[FaqsFaqs]:
        """Ripple Energy Frequently Asked Questions

        Do not pass the tag parameter to display all faqs"""
        if not tag:
            logger.info("Querying all FAQs.")
        else:
            logger.info(f"Querying FAQs with tag: {tag}")

        data = await self.client.faqs(tag=tag)

        logger.debug(f"FAQs response: {data}")

        return data

    @check_expiry
    async def wind_farm_generation(self) -> WindFarmGenerationMember | None:
        """Ripple Energy wind farm generation"""
        data = await self.client.wind_farm_generation()

        logger.debug(f"Wind farm generation response: {data}")

        return data

    @check_expiry
    @validate_call
    async def monthly_savings(
        self, date: datetime = datetime.now()
    ) -> MonthlySavingsMonthlySavingsData | None:
        """Ripple Energy monthly savings

        Do not pass the date parameter to display from today"""
        date_str: str = date.strftime("%Y-%m-%d")

        logger.info(f"Querying monthly savings for date: {date_str}")

        data = await self.client.monthly_savings(date=date_str)

        logger.debug(f"Monthly savings response: {data}")

        return data

    @check_expiry
    async def cumulative_savings(self) -> CumulativeSavingsCumulativeSavingsData | None:
        """Ripple Energy cumulative savings"""
        data = await self.client.cumulative_savings()

        logger.debug(f"Cumulative savings response: {data}")

        return data

    @check_expiry
    @validate_call
    async def coop_timeline_progression(
        self, coop_code: str
    ) -> CoopTimelineProgressionCoopTimelineProgression:
        """Ripple Energy co-op timeline progression"""
        data = await self.client.coop_timeline_progression(coop_code)

        logger.debug(f"Co-op timeline progression response: {data}")

        return data

    @check_expiry
    async def consumption(self) -> ConsumptionConsumption | None:
        """Ripple Energy estimated household consumption"""
        data = await self.client.consumption()

        logger.debug(f"Consumption response: {data}")

        return data

    @check_expiry
    async def all_coops(self) -> List[AllCoopsAllCoops] | None:
        """Ripple Energy all co-op's"""
        data = await self.client.all_coops()

        logger.debug(f"All co-op's response: {data}")

        return data



class RippleEnergyException(Exception):
    """Generic Ripple Energy Exception"""


class RippleEnergyMissingAuthorizationHeaderException(RippleEnergyException):
    """Ripple Energy Missing Authorization Header Exception"""


class RippleEnergyAuthenticationException(RippleEnergyException):
    """Ripple Energy Authentication Exception"""


class RippleEnergyDeauthenticationException(RippleEnergyException):
    """Ripple Energy Deauthentication Exception"""


class RippleEnergyTokenDestroyException(RippleEnergyException):
    """Ripple Energy Unable To Destroy Token Exception"""


class RippleEnergyTokenExpiredException(RippleEnergyException):
    """Ripple Energy Token Expired Exception"""



from io import IOBase

from pydantic import BaseModel as PydanticBaseModel
from pydantic import ConfigDict


class UnsetType:
    def __bool__(self) -> bool:
        return False


UNSET = UnsetType()


class BaseModel(PydanticBaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        arbitrary_types_allowed=True,
        protected_namespaces=(),
    )


class Upload:
    def __init__(self, filename: str, content: IOBase, content_type: str):
        self.filename = filename
        self.content = content
        self.content_type = content_type



from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class CumulativeSavings(BaseModel):
    cumulative_savings_data: Optional["CumulativeSavingsCumulativeSavingsData"] = Field(
        alias="cumulativeSavingsData"
    )


class CumulativeSavingsCumulativeSavingsData(BaseModel):
    currency: "CumulativeSavingsCumulativeSavingsDataCurrency"
    savings: float
    generation: float
    co2_saved: float = Field(alias="co2Saved")
    last_update: str = Field(alias="lastUpdate")


class CumulativeSavingsCumulativeSavingsDataCurrency(BaseModel):
    code: str
    precision: int


CumulativeSavings.model_rebuild()
CumulativeSavingsCumulativeSavingsData.model_rebuild()
CumulativeSavingsCumulativeSavingsDataCurrency.model_rebuild()



from .exceptions import (
    RippleEnergyAuthenticationException,
    RippleEnergyDeauthenticationException,
    RippleEnergyException,
    RippleEnergyMissingAuthorizationHeaderException,
    RippleEnergyTokenDestroyException,
    RippleEnergyTokenExpiredException,
)
from .models import RippleEnergyCredentialAuth, RippleEnergyTokenAuth
from .ripple_energy import RippleEnergy

__all__ = [
    "RippleEnergy",
    "RippleEnergyException",
    "RippleEnergyMissingAuthorizationHeaderException",
    "RippleEnergyAuthenticationException",
    "RippleEnergyDeauthenticationException",
    "RippleEnergyTokenDestroyException",
    "RippleEnergyTokenExpiredException",
    "RippleEnergyCredentialAuth",
    "RippleEnergyTokenAuth",
]



from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import GenerationDataTitle, GenerationGenerationFarmOperationalStatusChoices


class WindFarmGeneration(BaseModel):
    member: Optional["WindFarmGenerationMember"]


class WindFarmGenerationMember(BaseModel):
    id: str
    memberships: List["WindFarmGenerationMemberMemberships"]


class WindFarmGenerationMemberMemberships(BaseModel):
    id: str
    capacity: Any
    coop: "WindFarmGenerationMemberMembershipsCoop"


class WindFarmGenerationMemberMembershipsCoop(BaseModel):
    id: str
    first_year_estimated_bill_savings_per_watt_hour: float = Field(
        alias="firstYearEstimatedBillSavingsPerWattHour"
    )
    currency: "WindFarmGenerationMemberMembershipsCoopCurrency"
    generationfarm: "WindFarmGenerationMemberMembershipsCoopGenerationfarm"


class WindFarmGenerationMemberMembershipsCoopCurrency(BaseModel):
    precision: int
    code: str
    symbol: str


class WindFarmGenerationMemberMembershipsCoopGenerationfarm(BaseModel):
    id: str
    name: str
    capacity: Any
    operational_status: GenerationGenerationFarmOperationalStatusChoices = Field(
        alias="operationalStatus"
    )
    generation_data: List[
        "WindFarmGenerationMemberMembershipsCoopGenerationfarmGenerationData"
    ] = Field(alias="generationData")


class WindFarmGenerationMemberMembershipsCoopGenerationfarmGenerationData(BaseModel):
    title: GenerationDataTitle
    data_set: List[
        "WindFarmGenerationMemberMembershipsCoopGenerationfarmGenerationDataDataSet"
    ] = Field(alias="dataSet")


class WindFarmGenerationMemberMembershipsCoopGenerationfarmGenerationDataDataSet(
    BaseModel
):
    net_power_output_kwh: float = Field(alias="netPowerOutputKwh")
    date_time: str = Field(alias="dateTime")
    is_forecast_data: bool = Field(alias="isForecastData")
    savings_for_period: float = Field(alias="savingsForPeriod")


WindFarmGeneration.model_rebuild()
WindFarmGenerationMember.model_rebuild()
WindFarmGenerationMemberMemberships.model_rebuild()
WindFarmGenerationMemberMembershipsCoop.model_rebuild()
WindFarmGenerationMemberMembershipsCoopCurrency.model_rebuild()
WindFarmGenerationMemberMembershipsCoopGenerationfarm.model_rebuild()
WindFarmGenerationMemberMembershipsCoopGenerationfarmGenerationData.model_rebuild()
WindFarmGenerationMemberMembershipsCoopGenerationfarmGenerationDataDataSet.model_rebuild()



from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class Deauthenticate(BaseModel):
    auth_logout_session: Optional["DeauthenticateAuthLogoutSession"] = Field(
        alias="authLogoutSession"
    )
    delete_token_cookie: Optional["DeauthenticateDeleteTokenCookie"] = Field(
        alias="deleteTokenCookie"
    )
    delete_refresh_token_cookie: Optional[
        "DeauthenticateDeleteRefreshTokenCookie"
    ] = Field(alias="deleteRefreshTokenCookie")


class DeauthenticateAuthLogoutSession(BaseModel):
    logout_successful: Optional[bool] = Field(alias="logoutSuccessful")


class DeauthenticateDeleteTokenCookie(BaseModel):
    deleted: bool


class DeauthenticateDeleteRefreshTokenCookie(BaseModel):
    deleted: bool


Deauthenticate.model_rebuild()
DeauthenticateAuthLogoutSession.model_rebuild()
DeauthenticateDeleteTokenCookie.model_rebuild()
DeauthenticateDeleteRefreshTokenCookie.model_rebuild()



from typing import Any, Optional

from pydantic import Field

from .base_model import BaseModel


class RefreshToken(BaseModel):
    refresh_token: Optional["RefreshTokenRefreshToken"] = Field(alias="refreshToken")


class RefreshTokenRefreshToken(BaseModel):
    payload: Any
    refresh_expires_in: int = Field(alias="refreshExpiresIn")
    token: str


RefreshToken.model_rebuild()
RefreshTokenRefreshToken.model_rebuild()



from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel, Upload
from .enums import ApprovalStatus, Period


class InsightsChartDataInput(BaseModel):
    gen_farm_id: str = Field(alias="genFarmId")
    start_date: str = Field(alias="startDate")
    end_date: str = Field(alias="endDate")
    period: Period


class GetMemberMissingInvoiceDataInputs(BaseModel):
    member_id: str = Field(alias="memberId")
    coop_id: str = Field(alias="coopId")


class PaginationInput(BaseModel):
    offset: int
    limit: int


class SearchInput(BaseModel):
    offset: int
    limit: int
    search_term: str = Field(alias="searchTerm")


class SearchCountInput(BaseModel):
    search_term: str = Field(alias="searchTerm")


class SearchAndFilterInput(BaseModel):
    search: "SearchInput"
    filter: "FilterInput"


class FilterInput(BaseModel):
    filter_by_business_members: bool = Field(alias="filterByBusinessMembers")
    filter_by_approval_status: Optional[ApprovalStatus] = Field(
        alias="filterByApprovalStatus", default=None
    )


class SearchCountAndFilterInput(BaseModel):
    search: "SearchCountInput"
    filter: "FilterInput"


class SupplierMemberInfoInput(BaseModel):
    api_key: Optional[str] = Field(alias="apiKey", default=None)
    member_account_id: Optional[str] = Field(alias="memberAccountId", default=None)


class CalculateOrderPaymentInput(BaseModel):
    balance_amount: Optional[int] = Field(alias="balanceAmount", default=None)
    currency: str
    voucher_codes: Optional[List[Optional[str]]] = Field(
        alias="voucherCodes", default=None
    )
    instalments: Optional[int] = None
    order_lines: List["CalculatePaymentLineInput"] = Field(alias="orderLines")


class CalculatePaymentLineInput(BaseModel):
    units: int
    amount: Optional[int] = None
    sku: str
    params: Optional[Any] = None


class CalculateUnitsForCostInput(BaseModel):
    currency: str
    instalments: Optional[int] = None
    cost: Optional[int] = None
    sku: Optional[str] = None


class CreateReservedUserInput(BaseModel):
    create_account: "AuthenticationCreateAccountInput" = Field(alias="createAccount")
    update_details: "RegistrationUpdateUserPersonalDetailsInput" = Field(
        alias="updateDetails"
    )
    create_address: "AddressInput" = Field(alias="createAddress")
    pay_order: "PayOrderInput" = Field(alias="payOrder")


class AuthenticationCreateAccountInput(BaseModel):
    email: str
    password: str
    path: str


class RegistrationUpdateUserPersonalDetailsInput(BaseModel):
    first_name: str = Field(alias="firstName")
    last_name: str = Field(alias="lastName")
    phone_number: str = Field(alias="phoneNumber")


class AddressInput(BaseModel):
    postcode: str
    line1: str
    line2: Optional[str] = None
    town: str
    county: Optional[str] = None
    country: Optional[str] = None
    mpan: Optional[str] = None
    mprn: Optional[str] = None
    manually_entered: bool = Field(alias="manuallyEntered")
    selected_from_dropdown: Optional["OctopusApiAddressInput"] = Field(
        alias="selectedFromDropdown", default=None
    )
    requires_manual_switch: Optional[bool] = Field(
        alias="requiresManualSwitch", default=None
    )
    has_prepaid_meter: Optional[bool] = Field(alias="hasPrepaidMeter", default=None)


class OctopusApiAddressInput(BaseModel):
    line1: Optional[str] = None
    line2: Optional[str] = None
    line3: Optional[str] = None
    town: Optional[str] = None
    county: Optional[str] = None
    postcode: Optional[str] = None
    mpans: List[str]
    mprn: Optional[str] = None
    gsp: Optional[str] = None
    economy7: Optional[bool] = None
    prepaid_electricity_meter: Optional[bool] = Field(
        alias="prepaidElectricityMeter", default=None
    )
    display: str


class PayOrderInput(BaseModel):
    payment: "PaymentInput"
    order: "OrderInput"


class PaymentInput(BaseModel):
    unique_payment_id: Optional[str] = Field(alias="uniquePaymentId", default=None)
    description: str
    total_amount: int = Field(alias="totalAmount")
    total_tax_amount: int = Field(alias="totalTaxAmount")
    pay_amount: int = Field(alias="payAmount")
    voucher_amount: int = Field(alias="voucherAmount")
    balance_credit_amount: int = Field(alias="balanceCreditAmount")
    balance_debit_amount: int = Field(alias="balanceDebitAmount")
    currency: str
    stripe_payment: Optional["StripePaymentInput"] = Field(
        alias="stripePayment", default=None
    )
    billing_address: Optional["BillingAddress"] = Field(
        alias="billingAddress", default=None
    )
    email: Optional[str] = None
    voucher_codes: Optional[List[Optional[str]]] = Field(
        alias="voucherCodes", default=None
    )


class StripePaymentInput(BaseModel):
    payment_method_id: Optional[str] = Field(alias="paymentMethodId", default=None)
    payment_intent_id: Optional[str] = Field(alias="paymentIntentId", default=None)


class BillingAddress(BaseModel):
    line1: str
    line2: Optional[str] = None
    town: str
    county: Optional[str] = None
    postcode: str
    country: Optional[str] = None


class OrderInput(BaseModel):
    currency: str
    lines: List["PaymentLineInput"]
    instalments: Optional["InstalmentInput"] = None


class PaymentLineInput(BaseModel):
    amount: int
    units: int
    tax_amount: int = Field(alias="taxAmount")
    sku: str
    description: Optional[str] = None
    params: Optional[Any] = None


class InstalmentInput(BaseModel):
    description: str
    amount: int
    tax_amount: int = Field(alias="taxAmount")
    instalment_dates: List[Any] = Field(alias="instalmentDates")
    instalments: int
    lines: List["PaymentLineInput"]


class CreateOwnershipUserInput(BaseModel):
    create_address: "AddressInput" = Field(alias="createAddress")
    create_supplier_quotes: "UpdateSupplierQuotesInput" = Field(
        alias="createSupplierQuotes"
    )
    create_consumption: "ConsumptionInput" = Field(alias="createConsumption")
    create_quote: "UpdateQuoteInput" = Field(alias="createQuote")
    create_account: "CreateAccountInput" = Field(alias="createAccount")
    pay_order: "PayOrderInput" = Field(alias="payOrder")


class UpdateSupplierQuotesInput(BaseModel):
    electricity_annual_standard_kwh: Optional[float] = Field(
        alias="electricityAnnualStandardKwh", default=None
    )
    has_economy7_meter: Optional[bool] = Field(alias="hasEconomy7Meter", default=None)
    electricity_annual_day_kwh: Optional[int] = Field(
        alias="electricityAnnualDayKwh", default=None
    )
    electricity_annual_night_kwh: Optional[int] = Field(
        alias="electricityAnnualNightKwh", default=None
    )
    switches_gas: Optional[bool] = Field(alias="switchesGas", default=None)
    gas_annual_kwh: Optional[int] = Field(alias="gasAnnualKwh", default=None)
    has_smart_meter: Optional[bool] = Field(alias="hasSmartMeter", default=None)
    postcode: Optional[str] = None
    is_switch_delayed: Optional[bool] = Field(alias="isSwitchDelayed", default=None)
    switch_disabled: Optional[bool] = Field(alias="switchDisabled", default=None)
    delayed_switch_date: Optional[Any] = Field(alias="delayedSwitchDate", default=None)
    requires_manual_switch: Optional[bool] = Field(
        alias="requiresManualSwitch", default=None
    )
    supplier: Optional[int] = None
    selected_quote_code: Optional[str] = Field(alias="selectedQuoteCode", default=None)
    is_already_with_partner_supplier: Optional[bool] = Field(
        alias="isAlreadyWithPartnerSupplier", default=None
    )
    is_already_with_branded_supplier: Optional[bool] = Field(
        alias="isAlreadyWithBrandedSupplier", default=None
    )
    wants_to_see_tariffs: Optional[bool] = Field(
        alias="wantsToSeeTariffs", default=None
    )
    switches_to_another_partner_supplier: Optional[bool] = Field(
        alias="switchesToAnotherPartnerSupplier", default=None
    )
    electricity_supplier_id: Optional[int] = Field(
        alias="electricitySupplierId", default=None
    )
    acknowledges_tariffs_will_be_different_in_the_future: Optional[bool] = Field(
        alias="acknowledgesTariffsWillBeDifferentInTheFuture", default=None
    )
    acknowledges_he_will_lose_savings_if_he_doesnt_switch_on_time: Optional[
        bool
    ] = Field(alias="acknowledgesHeWillLoseSavingsIfHeDoesntSwitchOnTime", default=None)
    acknowledges_he_needs_to_contact_the_supplier_to_update_address: Optional[
        bool
    ] = Field(
        alias="acknowledgesHeNeedsToContactTheSupplierToUpdateAddress", default=None
    )


class ConsumptionInput(BaseModel):
    electricity_annual: float = Field(alias="electricityAnnual")
    from_calculator: Optional[bool] = Field(alias="fromCalculator", default=None)
    people_count: Optional[int] = Field(alias="peopleCount", default=None)
    bedrooms_count: Optional[int] = Field(alias="bedroomsCount", default=None)
    has_electric_vehicle: Optional[bool] = Field(
        alias="hasElectricVehicle", default=None
    )
    has_electric_heating: Optional[bool] = Field(
        alias="hasElectricHeating", default=None
    )
    has_heat_pump: Optional[bool] = Field(alias="hasHeatPump", default=None)
    has_solar_panels: Optional[bool] = Field(alias="hasSolarPanels", default=None)
    solar_panels_capacity: Optional[float] = Field(
        alias="solarPanelsCapacity", default=None
    )
    acknowledges_to_provide_bill_evidence: Optional[bool] = Field(
        alias="acknowledgesToProvideBillEvidence", default=None
    )


class UpdateQuoteInput(BaseModel):
    electricity_consumption_annual: int = Field(alias="electricityConsumptionAnnual")
    voucher_codes: List[str] = Field(alias="voucherCodes")
    monetary_value: Optional[int] = Field(alias="monetaryValue", default=None)
    capacity: Optional[int] = None


class CreateAccountInput(BaseModel):
    user: "UserInput"
    is_business: bool = Field(alias="isBusiness")
    business_name: Optional[str] = Field(alias="businessName", default=None)


class UserInput(BaseModel):
    first_name: str = Field(alias="firstName")
    last_name: str = Field(alias="lastName")
    email: str
    phone_number: str = Field(alias="phoneNumber")
    password: str
    password_confirm: str = Field(alias="passwordConfirm")


class SingleFileUploadInput(BaseModel):
    file: Upload
    category: Optional[str] = None


class TokenAuthenticationInput(BaseModel):
    email: str
    password: str


class ClientEnvInput(BaseModel):
    store: List["GetKeyValuePair"]
    site_url: str = Field(alias="siteUrl")
    path: str
    query: str
    referrer_url: Optional[str] = Field(alias="referrerUrl", default=None)
    client_id: str = Field(alias="clientId")
    client_version: str = Field(alias="clientVersion")


class GetKeyValuePair(BaseModel):
    key: str
    value: Optional[str] = None
    expiry: Optional[str] = None


class PayScheduledPaymentInput(BaseModel):
    payment_id: Optional[int] = Field(alias="paymentId", default=None)


class PayCapacityQuoteInput(BaseModel):
    capacity_quote_id: int = Field(alias="capacityQuoteId")
    instalments: int


class UpdateUserInput(BaseModel):
    id: Optional[int] = None
    phone_number: Optional[str] = Field(alias="phoneNumber", default=None)


class ChangeUserPasswordInput(BaseModel):
    current_password: Optional[str] = Field(alias="currentPassword", default=None)
    new_password: Optional[str] = Field(alias="newPassword", default=None)
    confirm_new_password: Optional[str] = Field(
        alias="confirmNewPassword", default=None
    )


class PasswordResetConfirmInput(BaseModel):
    password: str
    password_confirm: str = Field(alias="passwordConfirm")
    token: str
    uidb64: str


class UpdateUserCRMInput(BaseModel):
    id: int
    email: str


class DirectDebitInput(BaseModel):
    account_name: str = Field(alias="accountName")
    account_sort_code: str = Field(alias="accountSortCode")
    account_number: str = Field(alias="accountNumber")
    payment_day: str = Field(alias="paymentDay")


class AuthLoginSessionInputType(BaseModel):
    email: str
    password: str


class CreateMemberConsumptionEvidenceSubmissionInput(BaseModel):
    files: List["ConsumptionEvidenceInput"]
    note: Optional[str] = None


class ConsumptionEvidenceInput(BaseModel):
    file: Upload
    consumption: int


class ApproveMemberConsumptionEvidenceSubmissionInput(BaseModel):
    submission_id: str = Field(alias="submissionId")
    approved_evidence: float = Field(alias="approvedEvidence")
    admin_note: Optional[str] = Field(alias="adminNote", default=None)


class UpdateMemberConsumptionEvidenceSubmissionAdminNoteInput(BaseModel):
    submission_id: str = Field(alias="submissionId")
    admin_note: str = Field(alias="adminNote")


class AddMemberToCoopWaitingListInput(BaseModel):
    coop_id: str = Field(alias="coopId")
    generation_requested_kwh: int = Field(alias="generationRequestedKwh")


class CreateMemberBeneficiaryInput(BaseModel):
    nominated_person: str = Field(alias="nominatedPerson")
    email: str
    phone: str
    line1: str
    line2: Optional[str] = None
    town: str
    postcode: str


class UpdateMemberBeneficiaryInput(BaseModel):
    beneficiary_id: str = Field(alias="beneficiaryId")
    nominated_person: str = Field(alias="nominatedPerson")
    email: str
    phone: str
    line1: str
    line2: Optional[str] = None
    town: str
    postcode: str


class ResponseInputType(BaseModel):
    question: str
    survey_response: Optional[str] = Field(alias="surveyResponse", default=None)
    email: Optional[str] = None
    multiple_choice_answer: Optional[List[Optional[str]]] = Field(
        alias="multipleChoiceAnswer", default=None
    )
    radio_choice_answer: Optional[str] = Field(alias="radioChoiceAnswer", default=None)
    text_field_answer: Optional[str] = Field(alias="textFieldAnswer", default=None)
    number_answer: Optional[int] = Field(alias="numberAnswer", default=None)
    yes_no_answer: Optional[str] = Field(alias="yesNoAnswer", default=None)
    survey_id: Optional[str] = Field(alias="surveyId", default=None)


class AddAdditionalWattsInputs(BaseModel):
    member_id: str = Field(alias="memberId")
    capacity: float
    coop_id: str = Field(alias="coopId")
    cost_of_watts: float = Field(alias="costOfWatts")
    cost_of_fees_net: float = Field(alias="costOfFeesNet")
    cost_of_fees_tax: float = Field(alias="costOfFeesTax")
    paid: bool
    add_invoice_only: bool = Field(alias="addInvoiceOnly")
    due: Optional[str] = None
    invoice_date: str = Field(alias="invoiceDate")
    external_invoice_url: Optional[str] = Field(
        alias="externalInvoiceUrl", default=None
    )


class CreateOrUpdateNewsInput(BaseModel):
    id: Optional[str] = None
    title: str
    body: str
    hidden: bool
    coop_id: Optional[str] = Field(alias="coopId", default=None)


class SingleNewsFileUploadInput(BaseModel):
    file: Upload
    category: str
    news_id: str = Field(alias="newsId")


class CreateUserMemoInputs(BaseModel):
    note: str
    user_id: str = Field(alias="userId")
    channel: str
    open: bool
    action_required: bool = Field(alias="actionRequired")


class CreateContactUsMessageInput(BaseModel):
    name: str
    email: str
    message: str


class UserContactInput(BaseModel):
    firstname: str
    lastname: str
    email: str


class CreateEmployerContactInput(BaseModel):
    firstname: str
    lastname: str
    email: str
    phone: Optional[str] = None
    company: str
    number_of_employees: Optional[str] = Field(alias="numberOfEmployees", default=None)
    current_energy_supplier: Optional[str] = Field(
        alias="currentEnergySupplier", default=None
    )
    annual_consumption_kwh: Optional[str] = Field(
        alias="annualConsumptionKwh", default=None
    )
    number_of_sites: Optional[str] = Field(alias="numberOfSites", default=None)
    message: Optional[str] = None
    pathname: Optional[str] = None
    job_title: Optional[str] = Field(alias="jobTitle", default=None)


class RequestCallBackInput(BaseModel):
    first_name: str = Field(alias="firstName")
    last_name: Optional[str] = Field(alias="lastName", default=None)
    phone_number: str = Field(alias="phoneNumber")
    company_name: Optional[str] = Field(alias="companyName", default=None)
    message: Optional[str] = None
    pathname: str
    number_of_employees: Optional[str] = Field(alias="numberOfEmployees", default=None)
    current_energy_supplier: Optional[str] = Field(
        alias="currentEnergySupplier", default=None
    )
    annual_consumption_kwh: Optional[str] = Field(
        alias="annualConsumptionKwh", default=None
    )
    number_of_sites: Optional[str] = Field(alias="numberOfSites", default=None)


class QuoteInput(BaseModel):
    cost_total: float = Field(alias="costTotal")
    percentage_of_consumption_covered: float = Field(
        alias="percentageOfConsumptionCovered"
    )
    first_year_bill_savings: float = Field(alias="firstYearBillSavings")
    project_lifespan_bill_savings: float = Field(alias="projectLifespanBillSavings")
    annual_co2_savings: float = Field(alias="annualCo2Savings")
    project_lifespan_in_years: int = Field(alias="projectLifespanInYears")


class GiftCardInput(BaseModel):
    amount: int
    delivery_date: str = Field(alias="deliveryDate")
    design_id: str = Field(alias="designId")
    message: Optional[str] = None
    sender: Optional["SenderInput"] = None
    recipient: "RecipientInput"
    is_send_post_card: Optional[bool] = Field(alias="isSendPostCard", default=None)


class SenderInput(BaseModel):
    first_name: Optional[str] = Field(alias="firstName", default=None)
    last_name: Optional[str] = Field(alias="lastName", default=None)
    is_anonymous: bool = Field(alias="isAnonymous")


class RecipientInput(BaseModel):
    first_name: str = Field(alias="firstName")
    last_name: str = Field(alias="lastName")
    email: str
    address: Optional["RecipientAddressInput"] = None


class RecipientAddressInput(BaseModel):
    line1: str
    line2: Optional[str] = None
    town: str
    postcode: str


InsightsChartDataInput.model_rebuild()
GetMemberMissingInvoiceDataInputs.model_rebuild()
PaginationInput.model_rebuild()
SearchInput.model_rebuild()
SearchCountInput.model_rebuild()
SearchAndFilterInput.model_rebuild()
FilterInput.model_rebuild()
SearchCountAndFilterInput.model_rebuild()
SupplierMemberInfoInput.model_rebuild()
CalculateOrderPaymentInput.model_rebuild()
CalculatePaymentLineInput.model_rebuild()
CalculateUnitsForCostInput.model_rebuild()
CreateReservedUserInput.model_rebuild()
AuthenticationCreateAccountInput.model_rebuild()
RegistrationUpdateUserPersonalDetailsInput.model_rebuild()
AddressInput.model_rebuild()
OctopusApiAddressInput.model_rebuild()
PayOrderInput.model_rebuild()
PaymentInput.model_rebuild()
StripePaymentInput.model_rebuild()
BillingAddress.model_rebuild()
OrderInput.model_rebuild()
PaymentLineInput.model_rebuild()
InstalmentInput.model_rebuild()
CreateOwnershipUserInput.model_rebuild()
UpdateSupplierQuotesInput.model_rebuild()
ConsumptionInput.model_rebuild()
UpdateQuoteInput.model_rebuild()
CreateAccountInput.model_rebuild()
UserInput.model_rebuild()
SingleFileUploadInput.model_rebuild()
TokenAuthenticationInput.model_rebuild()
ClientEnvInput.model_rebuild()
GetKeyValuePair.model_rebuild()
PayScheduledPaymentInput.model_rebuild()
PayCapacityQuoteInput.model_rebuild()
UpdateUserInput.model_rebuild()
ChangeUserPasswordInput.model_rebuild()
PasswordResetConfirmInput.model_rebuild()
UpdateUserCRMInput.model_rebuild()
DirectDebitInput.model_rebuild()
AuthLoginSessionInputType.model_rebuild()
CreateMemberConsumptionEvidenceSubmissionInput.model_rebuild()
ConsumptionEvidenceInput.model_rebuild()
ApproveMemberConsumptionEvidenceSubmissionInput.model_rebuild()
UpdateMemberConsumptionEvidenceSubmissionAdminNoteInput.model_rebuild()
AddMemberToCoopWaitingListInput.model_rebuild()
CreateMemberBeneficiaryInput.model_rebuild()
UpdateMemberBeneficiaryInput.model_rebuild()
ResponseInputType.model_rebuild()
AddAdditionalWattsInputs.model_rebuild()
CreateOrUpdateNewsInput.model_rebuild()
SingleNewsFileUploadInput.model_rebuild()
CreateUserMemoInputs.model_rebuild()
CreateContactUsMessageInput.model_rebuild()
UserContactInput.model_rebuild()
CreateEmployerContactInput.model_rebuild()
RequestCallBackInput.model_rebuild()
QuoteInput.model_rebuild()
GiftCardInput.model_rebuild()
SenderInput.model_rebuild()
RecipientInput.model_rebuild()
RecipientAddressInput.model_rebuild()



from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class Authenticate(BaseModel):
    token_auth: "AuthenticateTokenAuth" = Field(alias="tokenAuth")


class AuthenticateTokenAuth(BaseModel):
    token: Optional[str]


Authenticate.model_rebuild()
AuthenticateTokenAuth.model_rebuild()



from typing import List

from .base_model import BaseModel


class Faqs(BaseModel):
    faqs: List["FaqsFaqs"]


class FaqsFaqs(BaseModel):
    id: str
    question: str
    answer: str
    tags: List["FaqsFaqsTags"]


class FaqsFaqsTags(BaseModel):
    id: str
    name: str


Faqs.model_rebuild()
FaqsFaqs.model_rebuild()
FaqsFaqsTags.model_rebuild()



from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import (
    AccountType,
    CoopCoopStatusChoices,
    GenerationGenerationFarmGenerationTypeChoices,
    GenerationGenerationFarmOperationalStatusChoices,
)


class ConsumptionFragment(BaseModel):
    id: str
    electricity_annual_kwh: float = Field(alias="electricityAnnualKwh")
    from_calculator: bool = Field(alias="fromCalculator")
    people_count: int = Field(alias="peopleCount")
    bedrooms_count: int = Field(alias="bedroomsCount")
    has_electric_vehicle: bool = Field(alias="hasElectricVehicle")
    has_electric_heating: bool = Field(alias="hasElectricHeating")
    has_heat_pump: bool = Field(alias="hasHeatPump")
    has_solar_panels: bool = Field(alias="hasSolarPanels")
    solar_panels_capacity: float = Field(alias="solarPanelsCapacity")
    acknowledges_to_provide_bill_evidence: bool = Field(
        alias="acknowledgesToProvideBillEvidence"
    )


class CoopFragment(BaseModel):
    id: str
    name: str
    code: str
    wattage_sku: str = Field(alias="wattageSku")
    status: CoopCoopStatusChoices
    active: bool
    description: str
    max_generation_per_member: int = Field(alias="maxGenerationPerMember")
    min_generation_per_member: int = Field(alias="minGenerationPerMember")
    percentage_funded: int = Field(alias="percentageFunded")
    public_close_date: Optional[Any] = Field(alias="publicCloseDate")
    estimated_bill_savings_per_watt_hour: float = Field(
        alias="estimatedBillSavingsPerWattHour"
    )
    first_year_estimated_bill_savings_per_watt_hour: float = Field(
        alias="firstYearEstimatedBillSavingsPerWattHour"
    )
    cost_total_per_watt_hour: float = Field(alias="costTotalPerWattHour")
    is_available_for_local_purchase: bool = Field(alias="isAvailableForLocalPurchase")
    local_purchase_postcodes: Optional[List[Optional[str]]] = Field(
        alias="localPurchasePostcodes"
    )
    currency: "CoopFragmentCurrency"
    generationfarm: "CoopFragmentGenerationfarm"
    documents: List["CoopFragmentDocuments"]


class CoopFragmentCurrency(BaseModel):
    id: Optional[int]
    code: str
    precision: int
    symbol: str


class CoopFragmentGenerationfarm(BaseModel):
    id: str
    name: str
    latitude: Optional[float]
    longitude: Optional[float]
    is_location_confirmed: bool = Field(alias="isLocationConfirmed")
    start_date: Optional[Any] = Field(alias="startDate")
    operational_status: GenerationGenerationFarmOperationalStatusChoices = Field(
        alias="operationalStatus"
    )
    generation_type: GenerationGenerationFarmGenerationTypeChoices = Field(
        alias="generationType"
    )
    capacity: Any
    capacity_to_generation_factor: float = Field(alias="capacityToGenerationFactor")
    light_show_image: Optional[str] = Field(alias="lightShowImage")
    currency: "CoopFragmentGenerationfarmCurrency"


class CoopFragmentGenerationfarmCurrency(BaseModel):
    id: Optional[int]
    code: str
    precision: int
    symbol: str


class CoopFragmentDocuments(BaseModel):
    document_url: str = Field(alias="documentUrl")
    version: str
    document: "CoopFragmentDocumentsDocument"


class CoopFragmentDocumentsDocument(BaseModel):
    id: str
    name: str
    category: str
    subcategory: str
    description: str
    updated_at: Any = Field(alias="updatedAt")
    document_tags: List["CoopFragmentDocumentsDocumentDocumentTags"] = Field(
        alias="documentTags"
    )


class CoopFragmentDocumentsDocumentDocumentTags(BaseModel):
    id: str
    name: str


class WaitingListFragment(BaseModel):
    id: str
    coop: "WaitingListFragmentCoop"


class WaitingListFragmentCoop(BaseModel):
    id: str
    name: str


class WaitingListPlaceFragment(BaseModel):
    id: str
    generation_requested_kwh: int = Field(alias="generationRequestedKwh")
    waiting_list: "WaitingListPlaceFragmentWaitingList" = Field(alias="waitingList")


class WaitingListPlaceFragmentWaitingList(WaitingListFragment):
    pass


class MemberFragment(BaseModel):
    id: str
    is_business: bool = Field(alias="isBusiness")
    account_type: Optional[AccountType] = Field(alias="accountType")
    business_name: Optional[str] = Field(alias="businessName")
    business_registration_number: Optional[str] = Field(
        alias="businessRegistrationNumber"
    )
    registration_completed: bool = Field(alias="registrationCompleted")
    fully_charged_preregistered: bool = Field(alias="fullyChargedPreregistered")
    investment_total: int = Field(alias="investmentTotal")
    bill_savings_across_all_projects: int = Field(alias="billSavingsAcrossAllProjects")
    expected_generation_annual_total: Any = Field(alias="expectedGenerationAnnualTotal")
    ownership_percentage_total: float = Field(alias="ownershipPercentageTotal")
    capacity_owned_total: Any = Field(alias="capacityOwnedTotal")
    has_reserved_in_active_coop: bool = Field(alias="hasReservedInActiveCoop")
    date_of_reservation_in_active_coop: Optional[Any] = Field(
        alias="dateOfReservationInActiveCoop"
    )
    date_of_last_share_purchase: Optional[Any] = Field(alias="dateOfLastSharePurchase")
    has_bought_shares_in_active_coop: bool = Field(alias="hasBoughtSharesInActiveCoop")
    has_bought_shares_in_graig_fatha: bool = Field(alias="hasBoughtSharesInGraigFatha")
    has_bought_shares_in_kirk_hill: bool = Field(alias="hasBoughtSharesInKirkHill")
    has_bought_shares_in_multiple_coops: bool = Field(
        alias="hasBoughtSharesInMultipleCoops"
    )
    is_supplied_by_octopus_energy: bool = Field(alias="isSuppliedByOctopusEnergy")
    member_document: Optional["MemberFragmentMemberDocument"] = Field(
        alias="memberDocument"
    )
    address: Optional["MemberFragmentAddress"]
    badges: List["MemberFragmentBadges"]
    beneficiaries: List["MemberFragmentBeneficiaries"]
    invoices: List["MemberFragmentInvoices"]
    memberships: List["MemberFragmentMemberships"]
    current_member_supplier: Optional["MemberFragmentCurrentMemberSupplier"] = Field(
        alias="currentMemberSupplier"
    )
    waiting_list_places: Optional[
        List[Optional["MemberFragmentWaitingListPlaces"]]
    ] = Field(alias="waitingListPlaces")


class MemberFragmentMemberDocument(BaseModel):
    id: str
    file: str
    document_url: str = Field(alias="documentUrl")


class MemberFragmentAddress(BaseModel):
    id: str
    line1: str
    line2: str
    town: str
    postcode: str
    manually_entered: bool = Field(alias="manuallyEntered")


class MemberFragmentBadges(BaseModel):
    id: str
    name: str
    code: str
    description: str
    image_url: str = Field(alias="imageUrl")


class MemberFragmentBeneficiaries(BaseModel):
    id: str
    nominated_person: str = Field(alias="nominatedPerson")
    email: str
    phone: str
    line1: str
    line2: Optional[str]
    town: str
    postcode: str


class MemberFragmentInvoices(BaseModel):
    id: str
    category: str
    quote: Optional["MemberFragmentInvoicesQuote"]


class MemberFragmentInvoicesQuote(BaseModel):
    id: str
    arrangement_fee: int = Field(alias="arrangementFee")
    bill_savings_annual_estimate: int = Field(alias="billSavingsAnnualEstimate")
    capacity: Any
    cost_total: int = Field(alias="costTotal")
    cost_total_locale: Optional[str] = Field(alias="costTotalLocale")
    currency: "MemberFragmentInvoicesQuoteCurrency"
    ownership_multiplier: float = Field(alias="ownershipMultiplier")
    payment_total: int = Field(alias="paymentTotal")
    payment_total_locale: Optional[str] = Field(alias="paymentTotalLocale")
    electricity_consumption_annual: float = Field(alias="electricityConsumptionAnnual")


class MemberFragmentInvoicesQuoteCurrency(BaseModel):
    code: str
    precision: int
    symbol: str


class MemberFragmentMemberships(BaseModel):
    id: str
    capacity: Any
    reserved_capacity: Any = Field(alias="reservedCapacity")
    created_at: Any = Field(alias="createdAt")
    expected_generation_annual_total: Any = Field(alias="expectedGenerationAnnualTotal")
    ownership_percentage: float = Field(alias="ownershipPercentage")
    investment_total: int = Field(alias="investmentTotal")
    bill_savings_for_project_lifespan: int = Field(
        alias="billSavingsForProjectLifespan"
    )
    bill_savings_first_year_estimate_total: int = Field(
        alias="billSavingsFirstYearEstimateTotal"
    )
    first_ownership_purchase_date: Optional[Any] = Field(
        alias="firstOwnershipPurchaseDate"
    )
    coop: "MemberFragmentMembershipsCoop"
    light_show_image: Optional[str] = Field(alias="lightShowImage")


class MemberFragmentMembershipsCoop(CoopFragment):
    pass


class MemberFragmentCurrentMemberSupplier(BaseModel):
    id: str
    account_identifier: str = Field(alias="accountIdentifier")
    supplier: "MemberFragmentCurrentMemberSupplierSupplier"


class MemberFragmentCurrentMemberSupplierSupplier(BaseModel):
    id: str
    name: str
    logo_url: Optional[str] = Field(alias="logoUrl")


class MemberFragmentWaitingListPlaces(WaitingListPlaceFragment):
    pass


ConsumptionFragment.model_rebuild()
CoopFragment.model_rebuild()
CoopFragmentCurrency.model_rebuild()
CoopFragmentGenerationfarm.model_rebuild()
CoopFragmentGenerationfarmCurrency.model_rebuild()
CoopFragmentDocuments.model_rebuild()
CoopFragmentDocumentsDocument.model_rebuild()
CoopFragmentDocumentsDocumentDocumentTags.model_rebuild()
WaitingListFragment.model_rebuild()
WaitingListFragmentCoop.model_rebuild()
WaitingListPlaceFragment.model_rebuild()
WaitingListPlaceFragmentWaitingList.model_rebuild()
MemberFragment.model_rebuild()
MemberFragmentMemberDocument.model_rebuild()
MemberFragmentAddress.model_rebuild()
MemberFragmentBadges.model_rebuild()
MemberFragmentBeneficiaries.model_rebuild()
MemberFragmentInvoices.model_rebuild()
MemberFragmentInvoicesQuote.model_rebuild()
MemberFragmentInvoicesQuoteCurrency.model_rebuild()
MemberFragmentMemberships.model_rebuild()
MemberFragmentMembershipsCoop.model_rebuild()
MemberFragmentCurrentMemberSupplier.model_rebuild()
MemberFragmentCurrentMemberSupplierSupplier.model_rebuild()
MemberFragmentWaitingListPlaces.model_rebuild()



from typing import Dict, List, Optional, Union

from .active_coop_status import ActiveCoopStatus, ActiveCoopStatusCoop
from .all_coops import AllCoops, AllCoopsAllCoops
from .async_base_client import AsyncBaseClient
from .authenticate import Authenticate, AuthenticateTokenAuth
from .base_model import UNSET, UnsetType
from .consumption import Consumption, ConsumptionConsumption
from .coop import Coop, CoopCoop
from .coop_timeline_progression import (
    CoopTimelineProgression,
    CoopTimelineProgressionCoopTimelineProgression,
)
from .cumulative_savings import (
    CumulativeSavings,
    CumulativeSavingsCumulativeSavingsData,
)
from .deauthenticate import Deauthenticate
from .faqs import Faqs, FaqsFaqs
from .input_types import TokenAuthenticationInput
from .me import Me, MeMe
from .member import Member, MemberMember
from .monthly_savings import MonthlySavings, MonthlySavingsMonthlySavingsData
from .refresh_token import RefreshToken, RefreshTokenRefreshToken
from .tribe_url import TribeUrl
from .verify_token import VerifyToken, VerifyTokenVerifyToken
from .version import Version
from .wind_farm_generation import WindFarmGeneration, WindFarmGenerationMember


def gql(q: str) -> str:
    return q


class Client(AsyncBaseClient):
    async def deauthenticate(self) -> Deauthenticate:
        query = gql(
            """
            mutation Deauthenticate {
              authLogoutSession {
                logoutSuccessful
              }
              deleteTokenCookie {
                deleted
              }
              deleteRefreshTokenCookie {
                deleted
              }
            }
            """
        )
        variables: Dict[str, object] = {}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return Deauthenticate.model_validate(data)

    async def all_coops(self) -> Optional[List[AllCoopsAllCoops]]:
        query = gql(
            """
            query AllCoops {
              allCoops {
                ...CoopFragment
              }
            }

            fragment CoopFragment on Coop {
              id
              name
              code
              wattageSku
              status
              active
              description
              maxGenerationPerMember
              minGenerationPerMember
              percentageFunded
              publicCloseDate
              estimatedBillSavingsPerWattHour
              firstYearEstimatedBillSavingsPerWattHour
              costTotalPerWattHour
              isAvailableForLocalPurchase
              localPurchasePostcodes
              currency {
                id
                code
                precision
                symbol
              }
              generationfarm {
                id
                name
                latitude
                longitude
                isLocationConfirmed
                startDate
                operationalStatus
                generationType
                capacity
                capacityToGenerationFactor
                lightShowImage
                currency {
                  id
                  code
                  precision
                  symbol
                }
              }
              documents {
                documentUrl
                version
                document {
                  id
                  name
                  category
                  subcategory
                  description
                  updatedAt
                  documentTags {
                    id
                    name
                  }
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return AllCoops.model_validate(data).all_coops

    async def consumption(self) -> Optional[ConsumptionConsumption]:
        query = gql(
            """
            query Consumption {
              consumption {
                ...ConsumptionFragment
              }
            }

            fragment ConsumptionFragment on MemberConsumption {
              id
              electricityAnnualKwh
              fromCalculator
              peopleCount
              bedroomsCount
              hasElectricVehicle
              hasElectricHeating
              hasHeatPump
              hasSolarPanels
              solarPanelsCapacity
              acknowledgesToProvideBillEvidence
            }
            """
        )
        variables: Dict[str, object] = {}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return Consumption.model_validate(data).consumption

    async def coop(self) -> Optional[CoopCoop]:
        query = gql(
            """
            query Coop {
              coop {
                ...CoopFragment
              }
            }

            fragment CoopFragment on Coop {
              id
              name
              code
              wattageSku
              status
              active
              description
              maxGenerationPerMember
              minGenerationPerMember
              percentageFunded
              publicCloseDate
              estimatedBillSavingsPerWattHour
              firstYearEstimatedBillSavingsPerWattHour
              costTotalPerWattHour
              isAvailableForLocalPurchase
              localPurchasePostcodes
              currency {
                id
                code
                precision
                symbol
              }
              generationfarm {
                id
                name
                latitude
                longitude
                isLocationConfirmed
                startDate
                operationalStatus
                generationType
                capacity
                capacityToGenerationFactor
                lightShowImage
                currency {
                  id
                  code
                  precision
                  symbol
                }
              }
              documents {
                documentUrl
                version
                document {
                  id
                  name
                  category
                  subcategory
                  description
                  updatedAt
                  documentTags {
                    id
                    name
                  }
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return Coop.model_validate(data).coop

    async def member(self) -> Optional[MemberMember]:
        query = gql(
            """
            query Member {
              member {
                ...MemberFragment
              }
            }

            fragment CoopFragment on Coop {
              id
              name
              code
              wattageSku
              status
              active
              description
              maxGenerationPerMember
              minGenerationPerMember
              percentageFunded
              publicCloseDate
              estimatedBillSavingsPerWattHour
              firstYearEstimatedBillSavingsPerWattHour
              costTotalPerWattHour
              isAvailableForLocalPurchase
              localPurchasePostcodes
              currency {
                id
                code
                precision
                symbol
              }
              generationfarm {
                id
                name
                latitude
                longitude
                isLocationConfirmed
                startDate
                operationalStatus
                generationType
                capacity
                capacityToGenerationFactor
                lightShowImage
                currency {
                  id
                  code
                  precision
                  symbol
                }
              }
              documents {
                documentUrl
                version
                document {
                  id
                  name
                  category
                  subcategory
                  description
                  updatedAt
                  documentTags {
                    id
                    name
                  }
                }
              }
            }

            fragment MemberFragment on Member {
              id
              isBusiness
              accountType
              businessName
              businessRegistrationNumber
              registrationCompleted
              fullyChargedPreregistered
              investmentTotal
              billSavingsAcrossAllProjects
              expectedGenerationAnnualTotal
              ownershipPercentageTotal
              capacityOwnedTotal
              hasReservedInActiveCoop
              dateOfReservationInActiveCoop
              dateOfLastSharePurchase
              hasBoughtSharesInActiveCoop
              hasBoughtSharesInGraigFatha
              hasBoughtSharesInKirkHill
              hasBoughtSharesInMultipleCoops
              isSuppliedByOctopusEnergy
              memberDocument {
                id
                file
                documentUrl
              }
              address {
                id
                line1
                line2
                town
                postcode
                manuallyEntered
              }
              badges {
                id
                name
                code
                description
                imageUrl
              }
              beneficiaries {
                id
                nominatedPerson
                email
                phone
                line1
                line2
                town
                postcode
              }
              invoices {
                id
                category
                quote {
                  id
                  arrangementFee
                  billSavingsAnnualEstimate
                  capacity
                  costTotal
                  costTotalLocale
                  currency {
                    code
                    precision
                    symbol
                  }
                  ownershipMultiplier
                  paymentTotal
                  paymentTotalLocale
                  electricityConsumptionAnnual
                }
              }
              memberships {
                id
                capacity
                reservedCapacity
                createdAt
                expectedGenerationAnnualTotal
                ownershipPercentage
                investmentTotal
                billSavingsForProjectLifespan
                billSavingsFirstYearEstimateTotal
                firstOwnershipPurchaseDate
                coop {
                  ...CoopFragment
                }
                lightShowImage
              }
              currentMemberSupplier {
                id
                accountIdentifier
                supplier {
                  id
                  name
                  logoUrl
                }
              }
              waitingListPlaces {
                ...WaitingListPlaceFragment
              }
            }

            fragment WaitingListFragment on CoopWaitingList {
              id
              coop {
                id
                name
              }
            }

            fragment WaitingListPlaceFragment on MemberCoopWaitingListPlace {
              id
              generationRequestedKwh
              waitingList {
                ...WaitingListFragment
              }
            }
            """
        )
        variables: Dict[str, object] = {}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return Member.model_validate(data).member

    async def wind_farm_generation(self) -> Optional[WindFarmGenerationMember]:
        query = gql(
            """
            query WindFarmGeneration {
              member {
                id
                memberships {
                  id
                  capacity
                  coop {
                    id
                    firstYearEstimatedBillSavingsPerWattHour
                    currency {
                      precision
                      code
                      symbol
                    }
                    generationfarm {
                      id
                      name
                      capacity
                      operationalStatus
                      generationData {
                        title
                        dataSet {
                          netPowerOutputKwh
                          dateTime
                          isForecastData
                          savingsForPeriod
                        }
                      }
                    }
                  }
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return WindFarmGeneration.model_validate(data).member

    async def authenticate(
        self, input: TokenAuthenticationInput
    ) -> AuthenticateTokenAuth:
        query = gql(
            """
            mutation Authenticate($input: TokenAuthenticationInput!) {
              tokenAuth(input: $input) {
                token
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return Authenticate.model_validate(data).token_auth

    async def refresh_token(
        self, token: Union[Optional[str], UnsetType] = UNSET
    ) -> Optional[RefreshTokenRefreshToken]:
        query = gql(
            """
            mutation RefreshToken($token: String) {
              refreshToken(token: $token) {
                payload
                refreshExpiresIn
                token
              }
            }
            """
        )
        variables: Dict[str, object] = {"token": token}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return RefreshToken.model_validate(data).refresh_token

    async def verify_token(
        self, token: Union[Optional[str], UnsetType] = UNSET
    ) -> Optional[VerifyTokenVerifyToken]:
        query = gql(
            """
            mutation VerifyToken($token: String) {
              verifyToken(token: $token) {
                payload
              }
            }
            """
        )
        variables: Dict[str, object] = {"token": token}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return VerifyToken.model_validate(data).verify_token

    async def active_coop_status(self) -> Optional[ActiveCoopStatusCoop]:
        query = gql(
            """
            query ActiveCoopStatus {
              coop {
                id
                status
              }
            }
            """
        )
        variables: Dict[str, object] = {}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return ActiveCoopStatus.model_validate(data).coop

    async def coop_timeline_progression(
        self, coop_code: str
    ) -> CoopTimelineProgressionCoopTimelineProgression:
        query = gql(
            """
            query CoopTimelineProgression($coopCode: String!) {
              coopTimelineProgression(coopCode: $coopCode) {
                timelineProgression
              }
            }
            """
        )
        variables: Dict[str, object] = {"coopCode": coop_code}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return CoopTimelineProgression.model_validate(data).coop_timeline_progression

    async def cumulative_savings(
        self,
    ) -> Optional[CumulativeSavingsCumulativeSavingsData]:
        query = gql(
            """
            query CumulativeSavings {
              cumulativeSavingsData {
                currency {
                  code
                  precision
                }
                savings
                generation
                co2Saved
                lastUpdate
              }
            }
            """
        )
        variables: Dict[str, object] = {}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return CumulativeSavings.model_validate(data).cumulative_savings_data

    async def faqs(self, tag: str) -> List[FaqsFaqs]:
        query = gql(
            """
            query Faqs($tag: String!) {
              faqs(tag: $tag) {
                id
                question
                answer
                tags {
                  id
                  name
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"tag": tag}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return Faqs.model_validate(data).faqs

    async def me(self) -> Optional[MeMe]:
        query = gql(
            """
            query Me {
              me {
                id
                firstName
                lastName
                email
                phoneNumber
                isSuperuser
                isStaff
                isGuest
                isActive
                isEmailVerified
                dateJoined
                groups {
                  id
                  name
                  permissions {
                    id
                    codename
                    name
                  }
                }
                referred {
                  id
                  user {
                    id
                    firstName
                    lastName
                    dateJoined
                    member {
                      id
                    }
                  }
                  recommendedBy {
                    id
                  }
                }
                payments {
                  id
                  paid
                  amount
                }
                directDebit {
                  id
                  accountName
                  accountSortCode
                  accountNumber
                  paymentDay
                }
                member {
                  id
                  registrationCompleted
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return Me.model_validate(data).me

    async def monthly_savings(
        self, date: str
    ) -> Optional[MonthlySavingsMonthlySavingsData]:
        query = gql(
            """
            query MonthlySavings($date: String!) {
              monthlySavingsData(date: $date) {
                currency {
                  code
                  precision
                }
                month
                year
                savings
                referredSavings
                generation
                co2Saved
                treeEquivalent
              }
            }
            """
        )
        variables: Dict[str, object] = {"date": date}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return MonthlySavings.model_validate(data).monthly_savings_data

    async def tribe_url(self) -> str:
        query = gql(
            """
            query TribeUrl {
              tribeUrl
            }
            """
        )
        variables: Dict[str, object] = {}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return TribeUrl.model_validate(data).tribe_url

    async def version(self) -> Optional[str]:
        query = gql(
            """
            query Version {
              version
            }
            """
        )
        variables: Dict[str, object] = {}
        response = await self.execute(query=query, variables=variables)
        data = self.get_data(response)
        return Version.model_validate(data).version



from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class GetCoopTimelineProgression(BaseModel):
    coop_timeline_progression: "GetCoopTimelineProgressionCoopTimelineProgression" = (
        Field(alias="coopTimelineProgression")
    )


class GetCoopTimelineProgressionCoopTimelineProgression(BaseModel):
    timeline_progression: Optional[int] = Field(alias="timelineProgression")


GetCoopTimelineProgression.model_rebuild()
GetCoopTimelineProgressionCoopTimelineProgression.model_rebuild()



from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel


class Me(BaseModel):
    me: Optional["MeMe"]


class MeMe(BaseModel):
    id: str
    first_name: str = Field(alias="firstName")
    last_name: str = Field(alias="lastName")
    email: str
    phone_number: Optional[str] = Field(alias="phoneNumber")
    is_superuser: bool = Field(alias="isSuperuser")
    is_staff: bool = Field(alias="isStaff")
    is_guest: bool = Field(alias="isGuest")
    is_active: bool = Field(alias="isActive")
    is_email_verified: bool = Field(alias="isEmailVerified")
    date_joined: Any = Field(alias="dateJoined")
    groups: List["MeMeGroups"]
    referred: List["MeMeReferred"]
    payments: Optional[List["MeMePayments"]]
    direct_debit: Optional["MeMeDirectDebit"] = Field(alias="directDebit")
    member: Optional["MeMeMember"]


class MeMeGroups(BaseModel):
    id: str
    name: str
    permissions: List["MeMeGroupsPermissions"]


class MeMeGroupsPermissions(BaseModel):
    id: str
    codename: str
    name: str


class MeMeReferred(BaseModel):
    id: str
    user: "MeMeReferredUser"
    recommended_by: Optional["MeMeReferredRecommendedBy"] = Field(alias="recommendedBy")


class MeMeReferredUser(BaseModel):
    id: str
    first_name: str = Field(alias="firstName")
    last_name: str = Field(alias="lastName")
    date_joined: Any = Field(alias="dateJoined")
    member: Optional["MeMeReferredUserMember"]


class MeMeReferredUserMember(BaseModel):
    id: str


class MeMeReferredRecommendedBy(BaseModel):
    id: str


class MeMePayments(BaseModel):
    id: str
    paid: bool
    amount: int


class MeMeDirectDebit(BaseModel):
    id: str
    account_name: str = Field(alias="accountName")
    account_sort_code: str = Field(alias="accountSortCode")
    account_number: str = Field(alias="accountNumber")
    payment_day: str = Field(alias="paymentDay")


class MeMeMember(BaseModel):
    id: str
    registration_completed: bool = Field(alias="registrationCompleted")


Me.model_rebuild()
MeMe.model_rebuild()
MeMeGroups.model_rebuild()
MeMeGroupsPermissions.model_rebuild()
MeMeReferred.model_rebuild()
MeMeReferredUser.model_rebuild()
MeMeReferredUserMember.model_rebuild()
MeMeReferredRecommendedBy.model_rebuild()
MeMePayments.model_rebuild()
MeMeDirectDebit.model_rebuild()
MeMeMember.model_rebuild()



from typing import Optional

from .base_model import BaseModel
from .fragments import CoopFragment


class Coop(BaseModel):
    coop: Optional["CoopCoop"]


class CoopCoop(CoopFragment):
    pass


Coop.model_rebuild()
CoopCoop.model_rebuild()



from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class CoopTimelineProgression(BaseModel):
    coop_timeline_progression: "CoopTimelineProgressionCoopTimelineProgression" = Field(
        alias="coopTimelineProgression"
    )


class CoopTimelineProgressionCoopTimelineProgression(BaseModel):
    timeline_progression: Optional[int] = Field(alias="timelineProgression")


CoopTimelineProgression.model_rebuild()
CoopTimelineProgressionCoopTimelineProgression.model_rebuild()



from enum import Enum


class ApprovalStatus(str, Enum):
    APPROVAL_NOT_REQUIRED = "APPROVAL_NOT_REQUIRED"
    EVIDENCE_NEEDED = "EVIDENCE_NEEDED"
    EVIDENCE_PENDING_APPROVAL = "EVIDENCE_PENDING_APPROVAL"
    EVIDENCE_APPROVED = "EVIDENCE_APPROVED"


class MembersMemberConsumptionEvidenceSubmissionStatusChoices(str, Enum):
    EVIDENCE_NEEDED = "EVIDENCE_NEEDED"
    EVIDENCE_PENDING_APPROVAL = "EVIDENCE_PENDING_APPROVAL"
    EVIDENCE_APPROVED = "EVIDENCE_APPROVED"


class GenerationGenerationFarmGenerationTypeChoices(str, Enum):
    WIND = "WIND"
    SOL = "SOL"


class GenerationGenerationFarmOperationalStatusChoices(str, Enum):
    NA = "NA"
    PRELIMINARY = "PRELIMINARY"
    PLANNING = "PLANNING"
    CONTRACTED = "CONTRACTED"
    UNDER_CONSTRUCTION = "UNDER_CONSTRUCTION"
    OPERATIONAL = "OPERATIONAL"
    SUSPENDED = "SUSPENDED"
    DECOMMISSIONED = "DECOMMISSIONED"
    CLOSED = "CLOSED"


class GenerationDataTitle(str, Enum):
    TODAY = "TODAY"
    NEXT_4_DAYS = "NEXT_4_DAYS"
    NEXT_14_DAYS = "NEXT_14_DAYS"
    LAST_7_DAYS = "LAST_7_DAYS"
    LAST_30_DAYS = "LAST_30_DAYS"


class Period(str, Enum):
    Day = "Day"
    Week = "Week"
    Month = "Month"
    Year = "Year"


class CoopCoopStatusChoices(str, Enum):
    RESERVATIONS_OPEN = "RESERVATIONS_OPEN"
    RESERVATIONS_CLOSED = "RESERVATIONS_CLOSED"
    OWNERSHIP_OPEN_RESERVED_MEMBERS_ONLY = "OWNERSHIP_OPEN_RESERVED_MEMBERS_ONLY"
    OWNERSHIP_OPEN = "OWNERSHIP_OPEN"
    FULL = "FULL"
    BACKFILL = "BACKFILL"
    CLOSED = "CLOSED"


class AccountType(str, Enum):
    HOUSEHOLD = "HOUSEHOLD"
    SMALL_BUSINESS = "SMALL_BUSINESS"
    MEDIUM_BUSINESS = "MEDIUM_BUSINESS"
    LARGE_BUSINESS = "LARGE_BUSINESS"


class BusinessSector(str, Enum):
    ACCOUNTING = "ACCOUNTING"
    ADMINISTRATION_AND_OFFICE_SUPPORT = "ADMINISTRATION_AND_OFFICE_SUPPORT"
    ADVERTISING_AND_ARTS_AND_MEDIA = "ADVERTISING_AND_ARTS_AND_MEDIA"
    BANKING_AND_FINANCIAL_SERVICES = "BANKING_AND_FINANCIAL_SERVICES"
    CALL_CENTRE_AND_CUSTOMER_SERVICE = "CALL_CENTRE_AND_CUSTOMER_SERVICE"
    COMMUNITY_SERVICES_AND_DEVELOPMENT = "COMMUNITY_SERVICES_AND_DEVELOPMENT"
    CONSTRUCTION = "CONSTRUCTION"
    CONSULTING_AND_STRATEGY = "CONSULTING_AND_STRATEGY"
    DESIGN_AND_ARCHITECTURE = "DESIGN_AND_ARCHITECTURE"
    EDUCATION_AND_TRAINING = "EDUCATION_AND_TRAINING"
    ENGINEERING = "ENGINEERING"
    FARMING_AND_ANIMALS_AND_CONSERVATION = "FARMING_AND_ANIMALS_AND_CONSERVATION"
    GOVERNMENT_AND_DEFENCE = "GOVERNMENT_AND_DEFENCE"
    HEALTHCARE_AND_MEDICAL = "HEALTHCARE_AND_MEDICAL"
    HOSPITALITY_AND_TOURISM = "HOSPITALITY_AND_TOURISM"
    HUMAN_RESOURCES_AND_RECRUITMENT = "HUMAN_RESOURCES_AND_RECRUITMENT"
    INFORMATION_AND_COMMUNICATION_TECHNOLOGY = (
        "INFORMATION_AND_COMMUNICATION_TECHNOLOGY"
    )
    INSURANCE_AND_SUPERANNUATION = "INSURANCE_AND_SUPERANNUATION"
    LEGAL = "LEGAL"
    MANUFACTURING_AND_TRANSPORT_AND_LOGISTICS = (
        "MANUFACTURING_AND_TRANSPORT_AND_LOGISTICS"
    )
    MARKETING_AND_COMMUNICATIONS = "MARKETING_AND_COMMUNICATIONS"
    MINING_AND_RESOURCES_AND_ENERGY = "MINING_AND_RESOURCES_AND_ENERGY"
    REAL_ESTATE_AND_PROPERTY = "REAL_ESTATE_AND_PROPERTY"
    RETAIL_AND_CONSUMER_PRODUCTS = "RETAIL_AND_CONSUMER_PRODUCTS"
    SALES = "SALES"
    SCIENCE_AND_TECHNOLOGY = "SCIENCE_AND_TECHNOLOGY"
    SPORT_AND_RECREATION = "SPORT_AND_RECREATION"
    TRADES_AND_SERVICES = "TRADES_AND_SERVICES"
    OTHER = "OTHER"


class BusinessType(str, Enum):
    SOLE_TRADER = "SOLE_TRADER"
    PARTNERSHIP = "PARTNERSHIP"
    LIMITED_LIABILITY_PARTNERSHIP = "LIMITED_LIABILITY_PARTNERSHIP"
    LIMITED_COMPANY = "LIMITED_COMPANY"
    SOCIAL_ENTERPRISE = "SOCIAL_ENTERPRISE"
    CHARITY = "CHARITY"
    PUBLIC_SECTOR = "PUBLIC_SECTOR"
    OTHER = "OTHER"


class PaymentsPaymentCategoryChoices(str, Enum):
    JOINING_FEE = "JOINING_FEE"
    GIFT_VOUCHER = "GIFT_VOUCHER"
    ORDER = "ORDER"


class InvoiceType(str, Enum):
    RESERVATION_RECEIPT = "RESERVATION_RECEIPT"
    WATTAGE_AMOUNT = "WATTAGE_AMOUNT"
    ARRANGEMENT_FEE = "ARRANGEMENT_FEE"
    GIFT_VOUCHER = "GIFT_VOUCHER"
    INVOICE = "INVOICE"


class PaymentType(str, Enum):
    RESERVATION_FEE = "RESERVATION_FEE"
    GIFT_VOUCHER = "GIFT_VOUCHER"
    MONTHLY_INSTALMENT = "MONTHLY_INSTALMENT"
    INITIAL_INSTALMENT = "INITIAL_INSTALMENT"
    ONE_TIME_PAYMENT = "ONE_TIME_PAYMENT"
    PAYMENT = "PAYMENT"


class PaymentStatus(str, Enum):
    PAID = "PAID"
    FAILED = "FAILED"
    REFUNDED = "REFUNDED"
    UPCOMING = "UPCOMING"
    OVERDUE = "OVERDUE"


class CrmUserMemoChannelChoices(str, Enum):
    DIRECT_EMAIL = "DIRECT_EMAIL"
    INFO = "INFO"
    HELP = "HELP"
    CONTACT_US = "CONTACT_US"
    EMPLOYER = "EMPLOYER"
    BUSINESS = "BUSINESS"
    OTHER = "OTHER"


class GetMemberOctopusConsumptionDataTitle(str, Enum):
    LAST_7_DAYS = "LAST_7_DAYS"
    LAST_30_DAYS = "LAST_30_DAYS"


class SurveyQuestionChoiceTypeChoices(str, Enum):
    MULTIPLE_CHOICE = "MULTIPLE_CHOICE"
    YES_NO = "YES_NO"
    RADIO = "RADIO"
    TEXT = "TEXT"
    NUMBER = "NUMBER"


class StripePaymentType(str, Enum):
    CARD = "CARD"


class CardBrandType(str, Enum):
    OTHER = "OTHER"
    VISA = "VISA"
    MASTERCARD = "MASTERCARD"



from typing import Any, Optional

from pydantic import Field

from .base_model import BaseModel


class VerifyToken(BaseModel):
    verify_token: Optional["VerifyTokenVerifyToken"] = Field(alias="verifyToken")


class VerifyTokenVerifyToken(BaseModel):
    payload: Any


VerifyToken.model_rebuild()
VerifyTokenVerifyToken.model_rebuild()



from typing import Any, Optional

from pydantic import Field

from .base_model import BaseModel


class Verify(BaseModel):
    verify_token: Optional["VerifyVerifyToken"] = Field(alias="verifyToken")


class VerifyVerifyToken(BaseModel):
    payload: Any


Verify.model_rebuild()
VerifyVerifyToken.model_rebuild()



from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import GenerationDataTitle, GenerationGenerationFarmOperationalStatusChoices


class WindFarmGenerationData(BaseModel):
    member: Optional["WindFarmGenerationDataMember"]


class WindFarmGenerationDataMember(BaseModel):
    id: str
    memberships: List["WindFarmGenerationDataMemberMemberships"]


class WindFarmGenerationDataMemberMemberships(BaseModel):
    id: str
    capacity: Any
    coop: "WindFarmGenerationDataMemberMembershipsCoop"


class WindFarmGenerationDataMemberMembershipsCoop(BaseModel):
    id: str
    first_year_estimated_bill_savings_per_watt_hour: float = Field(
        alias="firstYearEstimatedBillSavingsPerWattHour"
    )
    currency: "WindFarmGenerationDataMemberMembershipsCoopCurrency"
    generationfarm: "WindFarmGenerationDataMemberMembershipsCoopGenerationfarm"


class WindFarmGenerationDataMemberMembershipsCoopCurrency(BaseModel):
    precision: int
    code: str
    symbol: str


class WindFarmGenerationDataMemberMembershipsCoopGenerationfarm(BaseModel):
    id: str
    name: str
    capacity: Any
    operational_status: GenerationGenerationFarmOperationalStatusChoices = Field(
        alias="operationalStatus"
    )
    generation_data: List[
        "WindFarmGenerationDataMemberMembershipsCoopGenerationfarmGenerationData"
    ] = Field(alias="generationData")


class WindFarmGenerationDataMemberMembershipsCoopGenerationfarmGenerationData(
    BaseModel
):
    title: GenerationDataTitle
    data_set: List[
        "WindFarmGenerationDataMemberMembershipsCoopGenerationfarmGenerationDataDataSet"
    ] = Field(alias="dataSet")


class WindFarmGenerationDataMemberMembershipsCoopGenerationfarmGenerationDataDataSet(
    BaseModel
):
    net_power_output_kwh: float = Field(alias="netPowerOutputKwh")
    date_time: str = Field(alias="dateTime")
    is_forecast_data: bool = Field(alias="isForecastData")
    savings_for_period: float = Field(alias="savingsForPeriod")


WindFarmGenerationData.model_rebuild()
WindFarmGenerationDataMember.model_rebuild()
WindFarmGenerationDataMemberMemberships.model_rebuild()
WindFarmGenerationDataMemberMembershipsCoop.model_rebuild()
WindFarmGenerationDataMemberMembershipsCoopCurrency.model_rebuild()
WindFarmGenerationDataMemberMembershipsCoopGenerationfarm.model_rebuild()
WindFarmGenerationDataMemberMembershipsCoopGenerationfarmGenerationData.model_rebuild()
WindFarmGenerationDataMemberMembershipsCoopGenerationfarmGenerationDataDataSet.model_rebuild()



from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel
from .fragments import CoopFragment


class AllCoops(BaseModel):
    all_coops: Optional[List["AllCoopsAllCoops"]] = Field(alias="allCoops")


class AllCoopsAllCoops(CoopFragment):
    pass


AllCoops.model_rebuild()
AllCoopsAllCoops.model_rebuild()



import logging
from datetime import datetime
from typing import Awaitable, Callable

from pydantic import validate_call

from .constants import JWT_HEADER_PREFIX, PARAM, TYPE
from .exceptions import RippleEnergyTokenExpiredException

logger = logging.getLogger(__name__)


@validate_call
def generate_jwt_header(token: str | None = None) -> dict[str, str]:
    return {"Authorization": f"{JWT_HEADER_PREFIX}{token}"}


@validate_call
def check_date(date: datetime, auto_auth_deauth: bool) -> bool:
    now = datetime.now()
    expired = date < now

    logger.info(f"Checked Token Expiry: {date} Against Now: {now}. Result: {expired}")

    if expired and not auto_auth_deauth:
        raise RippleEnergyTokenExpiredException

    return expired


def check_expiry(
    function: Callable[PARAM, Awaitable[TYPE]]
) -> Callable[PARAM, Awaitable[TYPE]]:
    async def wrapper(*args: PARAM.args, **kwargs: PARAM.kwargs) -> TYPE:
        if (
            hasattr(args[0], "token_expires")
            and hasattr(args[0], "auto_auth_deauth")
            and hasattr(args[0], "refresh_token")
        ):
            if check_date(args[0].token_expires, args[0].auto_auth_deauth):
                await args[0].refresh_token()
        return await function(*args, **kwargs)

    return wrapper
